@@ -129,7 +129,7 @@ class ThumbSrv(object):
 
             msg = "cannot create video thumbnails because some of the required programs are not available: "
             msg += ", ".join(missing)
-            self.log(msg, c=1)
+            self.log(msg, c=3)
 
         t = threading.Thread(target=self.cleaner)
         t.daemon = True
@@ -316,10 +316,10 @@ class ThumbSrv(object):
             time.sleep(interval)
             for vol in self.vols:
                 vol += "/.hist/th"
-                self.log("cln {}/".format(vol))
+                self.log("\033[Jcln {}/\033[A".format(vol))
                 self.clean(vol)
 
-            self.log("cln ok")
+            self.log("\033[Jcln ok")
 
     def clean(self, vol):
         # self.log("cln {}".format(vol))
